package come.stormborntales.notevault.ui.screens

import android.content.Context
import android.content.Intent
import android.graphics.BitmapFactory
import android.graphics.ImageDecoder
import android.media.Image
import android.net.Uri
import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Add
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.ImageBitmap
import androidx.compose.ui.graphics.asImageBitmap
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.unit.dp
import come.stormborntales.notevault.FullscreenImageViewerActivity
import come.stormborntales.notevault.data.local.entity.NoteEntity
import come.stormborntales.notevault.data.model.NoteEntry
import come.stormborntales.notevault.ui.viewmodel.NoteViewModel
import java.io.InputStream
import androidx.core.net.toUri

fun loadImageBitmap(context: Context, uriString: String): ImageBitmap? {
    return try {
        val uri = uriString.toUri()
        val inputStream: InputStream? = context.contentResolver.openInputStream(uri)
        val bitmap = BitmapFactory.decodeStream(inputStream)
        bitmap?.asImageBitmap()
    } catch (e: Exception) {
        e.printStackTrace()
        null
    }
}

@Composable
fun NoteCard(note: NoteEntity) {
    val context = LocalContext.current
    val imageBitmap = remember(note.images) {
        note.images.firstOrNull()?.let { loadImageBitmap(context, it) }
    }

    Card(
        modifier = Modifier
            .fillMaxWidth()
            .padding(vertical = 8.dp),
        shape = RoundedCornerShape(16.dp),
        elevation = CardDefaults.cardElevation(defaultElevation = 4.dp),
    ) {
        Row(modifier = Modifier.padding(16.dp)) {
            // Linkes Vorschaubild
            imageBitmap?.let {
                Image(
                    bitmap = it,
                    contentDescription = "Vorschaubild",
                    modifier = Modifier
                        .size(120.dp)
                        .clip(RoundedCornerShape(12.dp))
                )
                Spacer(modifier = Modifier.width(16.dp))
            }

            // Rechte Info-Spalte
            Column(
                modifier = Modifier
                    .weight(1f)
                    .align(Alignment.CenterVertically)
            ) {
                Text(
                    text = note.title,
                    style = MaterialTheme.typography.titleMedium
                )

                note.composer?.let {
                    Text("von $it", style = MaterialTheme.typography.labelMedium)
                }

                Spacer(modifier = Modifier.height(4.dp))

                note.year?.let {
                    Text("Jahr: $it", style = MaterialTheme.typography.bodySmall)
                }

                note.genre?.let {
                    Text("Genre: $it", style = MaterialTheme.typography.bodySmall)
                }

                note.description?.let {
                    Text("Beschreibung: $it", style = MaterialTheme.typography.bodySmall, maxLines = 2)
                }

                Spacer(modifier = Modifier.height(8.dp))

                Row(
                    horizontalArrangement = Arrangement.spacedBy(8.dp)
                ) {
                    OutlinedButton(
                        onClick = {
                            val uris = ArrayList<Uri>();
                            note.images.forEach { uris.add(it.toUri()) }

                            val intent = Intent(context, FullscreenImageViewerActivity::class.java).apply {
                                putParcelableArrayListExtra("imageUris", ArrayList(uris))
                            }
                            context.startActivity(intent)
                        },
                        contentPadding = PaddingValues(horizontal = 12.dp, vertical = 4.dp)
                    ) {
                        Text("Anzeigen", style = MaterialTheme.typography.labelLarge)
                    }
                    OutlinedButton(
                        onClick = { /* TODO */ },
                        contentPadding = PaddingValues(horizontal = 12.dp, vertical = 4.dp)
                    ) {
                        Text("Bearbeiten", style = MaterialTheme.typography.labelLarge)
                    }
                    OutlinedButton(
                        onClick = { /* TODO */ },
                        colors = ButtonDefaults.outlinedButtonColors(
                            contentColor = MaterialTheme.colorScheme.error
                        ),
                        contentPadding = PaddingValues(horizontal = 12.dp, vertical = 4.dp)
                    ) {
                        Text("Löschen", style = MaterialTheme.typography.labelLarge)
                    }
                }
            }
        }
    }
}


@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun MainScreen(
    onAddNoteClicked: () -> Unit, // Übergib hier deine Logik für den Import
    viewModel: NoteViewModel
) {
    val notes by viewModel.notes.observeAsState(emptyList())
    Scaffold(
        topBar = {
            TopAppBar(
                title = { Text("Meine Noten") }
            )
        },
        floatingActionButton = {
            FloatingActionButton(
                onClick = onAddNoteClicked
            ) {
                Icon(Icons.Default.Add, contentDescription = "Neue Noten hinzufügen")
            }
        },
        floatingActionButtonPosition = FabPosition.End
    ) { innerPadding ->
        LazyColumn(
            contentPadding = innerPadding,
            modifier = Modifier
                .fillMaxSize()
                .padding(16.dp)
        ) {
            items(notes) { note ->
                NoteCard(note = note)
            }
        }
    }
}